Attribute VB_Name = "Module1"
Option Explicit
Public glPrevWndProc As Long

Public Const GWL_WNDPROC = (-4)
Public Const WM_EXITSIZEMOVE = &H232
Public Const WM_MOUSEMOVE = &H200
Public Const WM_LBUTTONDOWN = &H201
Public Const WM_NCLBUTTONDOWN = &HA1

'The SetWindowLong function changes an attribute of the specified window.
'Calling SetWindowLong with the GWL_WNDPROC index creates a subclass of
'the window class used to create the window. An application should not
'subclass a window created by another process. The SetWindowLong function
'creates the window subclass by changing the window procedure associated
'with a particular window, causing Windows to call the new window procedure
'instead of the previous one. An application must pass any messages not
'processed by the new window procedure to the previous window procedure by
'calling CallWindowProc. This allows the application to create a chain of
'window procedures.
'
'Parameters
'hwnd - Identifies the window to which the window belongs.
'
'nIndex - Set to GWL_WNDPROC to set a new address for the window procedure.
'
'dwNewLong - Address of the new window procedure.
Declare Function SetWindowLong Lib "user32" Alias "SetWindowLongA" (ByVal hwnd As Long, ByVal nIndex As Long, ByVal dwNewLong As Long) As Long

'The CallWindowProc function passes message information to the specified window
'procedure. Use this function for window subclassing. Usually, all windows
'with the same class share one window procedure. A subclass is a window or
'set of windows with the same class whose messages are intercepted and processed
'by another window procedure before being passed to the window procedure of the class.
'
'Parameters:
'lpPrevWndFunc - Pointer to the previous window procedure.
'                If this value is obtained by calling the GetWindowLong function
'                with the nIndex parameter set to GWL_WNDPROC, it is the address
'                of a window or a handle representing that address.
'
'hwnd - Identifies the window procedure to receive the message.
'
'Msg - Specifies the message.
'
'wParam - Depends on the value of the Msg parameter.
'
'lParam - Depends on the value of the Msg parameter.
Declare Function CallWindowProc Lib "user32" Alias "CallWindowProcA" (ByVal lpPrevWndFunc As Long, ByVal hwnd As Long, ByVal Msg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long

Public Sub pUnSubClass()
  Call SetWindowLong(Form1.hwnd, GWL_WNDPROC, glPrevWndProc)
End Sub

Public Function fSubClass() As Long
  fSubClass = SetWindowLong(Form1.hwnd, GWL_WNDPROC, AddressOf pMyWindowProc)
End Function

Public Function pMyWindowProc(ByVal hw As Long, ByVal uMsg As Long, ByVal wParam As Long, ByVal lParam As Long) As Long
'
' This is the callback procedure used when a message is received
' by this form.  The desired message is processed and all others are
' passed back to the original procedure associated with the form.
'
If Form1.optMsg(0).Value And uMsg = WM_MOUSEMOVE Then
    Debug.Print "Mouse Moved"
    Exit Function
End If

If Form1.optMsg(1).Value And uMsg = WM_EXITSIZEMOVE Then
    Debug.Print "Done Resizing"
    Exit Function
End If

If Form1.optMsg(2).Value And uMsg = WM_LBUTTONDOWN Then
    Debug.Print "Left Button Down on Form"
    Exit Function
End If

If Form1.optMsg(3).Value And uMsg = WM_NCLBUTTONDOWN Then
    Debug.Print "Left Button Down on Caption"
    Exit Function
End If

pMyWindowProc = CallWindowProc(glPrevWndProc, hw, uMsg, wParam, lParam)
End Function
